#!/usr/bin/env node
import { TezosToolkit } from '@mavrykdynamics/taquito';
import { execSync } from 'child_process';
import * as fs from 'fs';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

const proxyContract = (lambdaFunction) => {
    return `// ------------------------------------------------------------------------------
// Types
// ------------------------------------------------------------------------------

type setLambdaType is [@layout:comb] record [
    name                  : string;
    func_bytes            : bytes;
]
type farmTypeType is 
        Farm    of unit
    |   MFarm   of unit
type setFarmLambdaType is [@layout:comb] record [
    name                  : string;
    func_bytes            : bytes;
    farmType              : farmTypeType;
]

type updateMetadataType is [@layout:comb] record [
    metadataKey      : string;
    metadataHash     : bytes; 
]

type updateType is 
    |   Update of unit
    |   Remove of unit

type updateWhitelistContractsType is [@layout:comb] record [
    whitelistContractAddress  : address;
    updateType                : updateType;
]

type updateGeneralContractsType is [@layout:comb] record [
    generalContractName     : string;
    generalContractAddress  : address;
    updateType              : updateType;
]

type updateWhitelistTokenContractsType is [@layout:comb] record [
    tokenContractAddress  : address;
    updateType            : updateType;
]

type aggregatorUpdateConfigActionType is 
        ConfigDecimals                      of unit
    |   ConfigAlphaPercentPerThousand       of unit

    |   ConfigPercentOracleThreshold        of unit
    |   ConfigHeartbeatSeconds              of unit

    |   ConfigRewardAmountStakedMvn         of unit
    |   ConfigRewardAmountXtz               of unit

type aggregatorUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue  : nat; 
    updateConfigAction    : aggregatorUpdateConfigActionType;
]

type aggregatorFactoryUpdateConfigActionType is 
    | ConfigAggregatorNameMaxLength   of unit
    | Empty                           of unit

type aggregatorFactoryUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue  : nat; 
    updateConfigAction    : aggregatorFactoryUpdateConfigActionType;
]

type breakGlassUpdateConfigActionType is 
        ConfigThreshold               of unit
    |   ConfigActionExpiryDays        of unit
    |   ConfigCouncilNameMaxLength    of unit
    |   ConfigCouncilWebsiteMaxLength of unit
    |   ConfigCouncilImageMaxLength   of unit

type breakGlassUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue  : nat; 
    updateConfigAction    : breakGlassUpdateConfigActionType;
]

type councilUpdateConfigActionType is 
        ConfigThreshold                       of unit
    |   ConfigActionExpiryDays                of unit
    |   ConfigCouncilNameMaxLength            of unit
    |   ConfigCouncilWebsiteMaxLength         of unit
    |   ConfigCouncilImageMaxLength           of unit
    |   ConfigRequestTokenNameMaxLength       of unit
    |   ConfigRequestPurposeMaxLength         of unit

type councilUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue        : nat; 
    updateConfigAction          : councilUpdateConfigActionType;
]

type delegationUpdateConfigActionType is 
        ConfigMinimumStakedMvnBalance of unit
    |   ConfigDelegationRatio         of unit
    |   ConfigMaxSatellites           of unit
    |   ConfigSatNameMaxLength        of unit
    |   ConfigSatDescMaxLength        of unit
    |   ConfigSatImageMaxLength       of unit
    |   ConfigSatWebsiteMaxLength     of unit

type delegationUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : delegationUpdateConfigActionType;
]

type doormanUpdateConfigActionType is 
        ConfigMinMvnAmount          of unit
    |   Empty                       of unit

type doormanUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : doormanUpdateConfigActionType;
]

type emergencyUpdateConfigActionType is 
        ConfigDurationInMinutes         of unit
    |   ConfigRequiredFeeMumav          of unit
    |   ConfigStakedMvnPercentRequired  of unit
    |   ConfigMinStakedMvnForVoting     of unit
    |   ConfigMinStakedMvnToTrigger     of unit
    |   ConfigProposalTitleMaxLength    of unit
    |   ConfigProposalDescMaxLength     of unit

type emergencyUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue  : nat; 
    updateConfigAction    : emergencyUpdateConfigActionType;
]

type farmUpdateConfigActionType is 
        ConfigForceRewardFromTransfer of unit
    |   ConfigRewardPerBlock of unit
type farmUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : farmUpdateConfigActionType;
]

type farmFactoryUpdateConfigActionType is 
        ConfigFarmNameMaxLength of unit
    |   Empty                   of unit

type farmFactoryUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : farmFactoryUpdateConfigActionType;
]

type governanceUpdateConfigActionType is 
        ConfigSuccessReward               of unit
    |   ConfigCycleVotersReward           of unit
    |   ConfigMinProposalRoundVotePct     of unit
    |   ConfigMinQuorumPercentage         of unit
    |   ConfigMinYayVotePercentage        of unit
    |   ConfigProposeFeeMumav             of unit
    |   ConfigMaxProposalsPerSatellite    of unit
    |   ConfigBlocksPerProposalRound      of unit
    |   ConfigBlocksPerVotingRound        of unit
    |   ConfigBlocksPerTimelockRound      of unit
    |   ConfigDataTitleMaxLength          of unit
    |   ConfigProposalTitleMaxLength      of unit
    |   ConfigProposalDescMaxLength       of unit
    |   ConfigProposalInvoiceMaxLength    of unit
    |   ConfigProposalCodeMaxLength       of unit

type governanceUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : governanceUpdateConfigActionType;
]

type governanceFinancialUpdateConfigActionType is
    |   ConfigApprovalPercentage          of unit
    |   ConfigFinancialReqDurationDays    of unit

type governanceFinancialUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : governanceFinancialUpdateConfigActionType;
]

type governanceSatelliteUpdateConfigActionType is 
        ConfigApprovalPercentage          of unit
    |   ConfigActionDurationInDays        of unit
    |   ConfigPurposeMaxLength            of unit
    |   ConfigMaxActionsPerSatellite      of unit

type governanceSatelliteUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue  : nat; 
    updateConfigAction    : governanceSatelliteUpdateConfigActionType;
]

type lendingControllerUpdateConfigActionType is 
        ConfigCollateralRatio           of unit
    |   ConfigLiquidationRatio          of unit
    |   ConfigLiquidationFeePercent     of unit
    |   ConfigAdminLiquidationFee       of unit
    |   ConfigMinimumLoanFeePercent     of unit
    |   ConfigMinLoanFeeTreasuryShare   of unit
    |   ConfigInterestTreasuryShare     of unit
    |   ConfigLastCompletedDataMaxDelay of unit
    |   ConfigMaxVaultLiqPercent        of unit
    |   ConfigLiquidationDelayInMins    of unit
    |   ConfigLiquidationMaxDuration    of unit

type lendingControllerUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat;  
    updateConfigAction      : lendingControllerUpdateConfigActionType;
]

type treasuryFactoryUpdateConfigActionType is 
        ConfigTreasuryNameMaxLength of unit
    |   Empty                       of unit
type treasuryFactoryUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : treasuryFactoryUpdateConfigActionType;
]

type vaultFactoryUpdateConfigActionType is 
    |   ConfigVaultNameMaxLength of unit
    |   Empty                    of unit

type vaultFactoryUpdateConfigParamsType is [@layout:comb] record [
    updateConfigNewValue    : nat; 
    updateConfigAction      : vaultFactoryUpdateConfigActionType;
]

type aggregatorPausableEntrypointType is
        UpdateData                    of bool
    |   WithdrawRewardXtz             of bool
    |   WithdrawRewardStakedMvn       of bool

type aggregatorTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : aggregatorPausableEntrypointType;
    empty             : unit
];

type aggregatorFactoryPausableEntrypointType is
        CreateAggregator            of bool
    |   UntrackAggregator           of bool
    |   TrackAggregator             of bool
    |   DistributeRewardXtz         of bool
    |   DistributeRewardStakedMvn   of bool

type aggregatorFactoryTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint      : aggregatorFactoryPausableEntrypointType;
    empty                 : unit
];

type delegationPausableEntrypointType is
        DelegateToSatellite             of bool
    |   UndelegateFromSatellite         of bool
    |   RegisterAsSatellite             of bool
    |   UnregisterAsSatellite           of bool
    |   UpdateSatelliteRecord           of bool
    |   DistributeReward                of bool
    |   TakeSatellitesSnapshot          of bool

type delegationTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : delegationPausableEntrypointType;
    empty             : unit
];

type doormanPausableEntrypointType is
        StakeMvn                      of bool
    |   UnstakeMvn                    of bool
    |   Exit                          of bool
    |   Compound                      of bool
    |   FarmClaim                     of bool
    |   OnVaultDepositStake           of bool
    |   OnVaultWithdrawStake          of bool
    |   OnVaultLiquidateStake         of bool

type doormanTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : doormanPausableEntrypointType;
    empty             : unit
];

type farmPausableEntrypointType is
        Deposit     of bool
    |   Withdraw    of bool
    |   Claim       of bool

type farmTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : farmPausableEntrypointType;
    empty             : unit
];

type farmFactoryPausableEntrypointType is
        CreateFarm         of bool
    |   CreateFarmMToken   of bool
    |   UntrackFarm        of bool
    |   TrackFarm          of bool

type farmFactoryTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : farmFactoryPausableEntrypointType;
    empty             : unit
];

type lendingControllerPausableEntrypointType is

        // Lending Controller Admin Entrypoints
    |   SetLoanToken                of bool
    |   SetCollateralToken          of bool

        // Lending Controller Token Pool Entrypoints
    |   AddLiquidity                of bool
    |   RemoveLiquidity             of bool

        // Lending Controller Vault Entrypoints
    |   RegisterVaultCreation       of bool
    |   CloseVault                  of bool
    |   RegisterDeposit             of bool
    |   RegisterWithdrawal          of bool
    |   MarkForLiquidation          of bool
    |   LiquidateVault              of bool
    |   Borrow                      of bool
    |   Repay                       of bool

        // Vault Entrypoints
    |   VaultDeposit                of bool
    |   VaultWithdraw               of bool
    |   VaultOnLiquidate            of bool

        // Vault Staked Token Entrypoints
    |   VaultDepositStakedToken     of bool
    |   VaultWithdrawStakedToken    of bool

type lendingControllerTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : lendingControllerPausableEntrypointType;
    empty             : unit
];

type treasuryPausableEntrypointType is
        Transfer                       of bool   
    |   MintMvnAndTransfer             of bool
    |   UpdateTokenOperators           of bool
    |   StakeTokens                    of bool
    |   UnstakeTokens                  of bool

type treasuryTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : treasuryPausableEntrypointType;
    empty             : unit
];

type treasuryFactoryPausableEntrypointType is
        CreateTreasury         of bool
    |   TrackTreasury          of bool
    |   UntrackTreasury        of bool

type treasuryFactoryTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : treasuryFactoryPausableEntrypointType;
    empty             : unit
];

type vaultFactoryPausableEntrypointType is
    |   CreateVault         of bool  
    |   Empty               of unit 

type vaultFactoryTogglePauseEntrypointType is [@layout:comb] record [
    targetEntrypoint  : vaultFactoryPausableEntrypointType;
    empty             : unit
];

type lpStandardType is
        Fa12 of unit
    |   Fa2 of unit

type farmLpTokenType is [@layout:comb] record [
    tokenAddress             : address;
    tokenId                  : nat;
    tokenStandard            : lpStandardType;
]
type farmPlannedRewardsType is [@layout:comb] record[
    totalBlocks              : nat;
    currentRewardPerBlock    : nat;
]
type createFarmType is [@layout:comb] record[
    name                     : string;
    addToGeneralContracts    : bool;
    forceRewardFromTransfer  : bool;
    infinite                 : bool;
    plannedRewards           : farmPlannedRewardsType;
    metadata                 : bytes;
    lpToken                  : farmLpTokenType;
]
type createFarmMTokenType is [@layout:comb] record[
    name                     : string;
    loanToken                : string;
    addToGeneralContracts    : bool;
    forceRewardFromTransfer  : bool;
    infinite                 : bool;
    plannedRewards           : farmPlannedRewardsType;
    metadata                 : bytes;
    lpToken                  : farmLpTokenType;
]

type initFarmParamsType is [@layout:comb] record[
    totalBlocks                 : nat;
    currentRewardPerBlock       : nat;
    forceRewardFromTransfer     : bool;
    infinite                    : bool;
]

type createTreasuryType is [@layout:comb] record[
    baker                   : option(key_hash); 
    name                    : string;
    addToGeneralContracts   : bool;
    metadata                : bytes;
]

type mavType             is unit
type fa12TokenType       is address
type fa2TokenType        is [@layout:comb] record [
    tokenContractAddress    : address;
    tokenId                 : nat;
]
type tokenType is
    |   Mav    of mavType         // unit
    |   Fa12   of fa12TokenType   // address
    |   Fa2    of fa2TokenType    // record [ tokenContractAddress : address; tokenId : nat; ]

type transferDestinationType is [@layout:comb] record[
    to_       : address;
    amount    : nat;
    token     : tokenType;
]

type transferActionType is list(transferDestinationType);

type mintMvnAndTransferType is [@layout:comb] record [
    to_             : address;
    amt             : nat;
]

type operatorParameterType is [@layout:comb] record[
    owner       : address;
    operator    : address;
    token_id    : nat;
]
type updateOperatorVariantType is 
        Add_operator    of operatorParameterType
    |   Remove_operator of operatorParameterType
type updateOperatorsType is list(updateOperatorVariantType)
type updateTokenOperatorsType is [@layout:comb] record [
    tokenContractAddress    : address;
    updateOperators         : updateOperatorsType;
]
type stakeTokensType is [@layout:comb] record [
    contractAddress  : address;
    amount           : nat;
]
type unstakeTokensType is [@layout:comb] record [
    contractAddress  : address;
    amount           : nat;
]

type aggregatorConfigType is [@layout:comb] record [
    decimals                            : nat;
    alphaPercentPerThousand             : nat;

    percentOracleThreshold              : nat;
    heartbeatSeconds                    : nat;

    rewardAmountStakedMvn               : nat;
    rewardAmountXtz                     : nat;
];
type oracleInformationType is [@layout:comb] record [
    oraclePublicKey  : key;
    oraclePeerId     : string;
];
type oracleLedgerType            is map (address, oracleInformationType);
type createAggregatorParamsType is [@layout:comb] record[
    name                    : string;
    addToGeneralContracts   : bool;

    oracleLedger            : oracleLedgerType;
    
    aggregatorConfig        : aggregatorConfigType;
    metadata                : bytes;
];

type addVesteeType is [@layout:comb] record [
    vesteeAddress               : address;
    totalAllocatedAmount        : nat;
    cliffInMonths               : nat;
    vestingInMonths             : nat;
]

type updateVesteeType is [@layout:comb] record [
    vesteeAddress               : address;
    newTotalAllocatedAmount     : nat;
    newCliffInMonths            : nat;
    newVestingInMonths          : nat;
]

type createLoanTokenActionType is [@layout:comb] record [
    tokenName                               : string;
    tokenDecimals                           : nat;

    oracleAddress                           : address;

    mTokenAddress                           : address;

    reserveRatio                            : nat;  // percentage of token pool that should be kept as reserves for liquidity 
    optimalUtilisationRate                  : nat;  // kink point
    baseInterestRate                        : nat;  // base interest rate
    maxInterestRate                         : nat;  // max interest rate
    interestRateBelowOptimalUtilisation     : nat;  // interest rate below kink
    interestRateAboveOptimalUtilisation     : nat;  // interest rate above kink
    minRepaymentAmount                      : nat; 

    // variants at the end for taquito 
    tokenType                               : tokenType; 
]

type updateLoanTokenActionType is [@layout:comb] record [

    tokenName                               : string;

    oracleAddress                           : address;

    reserveRatio                            : nat;  // percentage of token pool that should be kept as reserves for liquidity 
    optimalUtilisationRate                  : nat;  // kink point
    baseInterestRate                        : nat;  // base interest rate
    maxInterestRate                         : nat;  // max interest rate
    interestRateBelowOptimalUtilisation     : nat;  // interest rate below kink
    interestRateAboveOptimalUtilisation     : nat;  // interest rate above kink
    minRepaymentAmount                      : nat; 

    isPaused                                : bool;
]

type setLoanTokenType is 
    |   CreateLoanToken      of createLoanTokenActionType
    |   UpdateLoanToken      of updateLoanTokenActionType

type setLoanTokenActionType is [@layout:comb] record [
    action      : setLoanTokenType;
    empty       : unit;
]

type createCollateralTokenActionType is [@layout:comb] record [
    tokenName               : string;
    tokenContractAddress    : address;
    tokenDecimals           : nat; 

    oracleAddress           : address;  
    protected               : bool;
    
    isScaledToken           : bool; // mToken
    
    // To extend functionality beyond sMVN to other staked tokens in future
    isStakedToken           : bool;
    stakingContractAddress  : option(address);

    maxDepositAmount        : option(nat);

    // variants at the end for taquito 
    tokenType               : tokenType; 
]

type updateCollateralTokenActionType is [@layout:comb] record [
    tokenName               : string;
    oracleAddress           : address;  
    isPaused                : bool;
    stakingContractAddress  : option(address);
    maxDepositAmount        : option(nat);
]

type setCollateralTokenType is 
    |   CreateCollateralToken      of createCollateralTokenActionType
    |   UpdateCollateralToken      of updateCollateralTokenActionType

type setCollateralTokenActionType is [@layout:comb] record [
    action      : setCollateralTokenType;
    empty       : unit;
]

type actionType is 
        // Default Entrypoint to Receive Mav
        Default                       of unit
    |   Empty                         of unit

const noOperations : list (operation) = nil;
type return is list (operation) * unit;

(* lamdda function *)
${lambdaFunction}

(* main entrypoint *)
function main (const action : actionType; const s : unit) : return is

    case action of [

            // Housekeeping Entrypoints
            Default (_parameters)                -> (lambdaFunction(), s)
        |   Empty (_parameters)                  -> ((nil : list(operation)), s)

    ]

;`;
};
const setAdmin = (targetContract, newAdminAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${newAdminAddress}": address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setAdmin",
            ("${targetContract}": address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_ADMIN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setGovernance = (targetContract, newGovernanceAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${newGovernanceAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setGovernance",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_GOVERNANCE_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setName = (targetContract, newName) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${newName}" : string),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setName",
            ("${targetContract}" : address)) : option(contract(string))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_NAME_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setLambda = (targetContract, lambdaName, lambdaBytes) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            name=("${lambdaName}" : string);
            func_bytes=("${lambdaBytes}": bytes)
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setLambda",
            ("${targetContract}" : address)) : option(contract(setLambdaType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_LAMBDA_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setProductLambda = (targetContract, lambdaName, lambdaBytes, farmType) => {
    // If the contract is a farm factory, the entrypoint type changes
    const entrypointType = farmType ? "setFarmLambdaType" : "setLambdaType";
    const additionalParameter = farmType ? `farmType=(${farmType}: farmTypeType);` : "";
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            name=("${lambdaName}" : string);
            func_bytes=("${lambdaBytes}": bytes);
            ${additionalParameter}
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setProductLambda",
            ("${targetContract}" : address)) : option(contract(${entrypointType}))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_PRODUCT_LAMBDA_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateMetadata = (targetContract, metadataKey, metadataHash) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            metadataKey=("${metadataKey}" : string);
            metadataHash=("${metadataHash}": bytes)
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateMetadata",
            ("${targetContract}" : address)) : option(contract(updateMetadataType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_METADATA_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateWhitelistContracts = (targetContract, whitelistContractAddress, updateType) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            whitelistContractAddress  = ("${whitelistContractAddress}" : address);
            updateType                = (${updateType} : updateType);
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateWhitelistContracts",
            ("${targetContract}" : address)) : option(contract(updateWhitelistContractsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_WHITELIST_CONTRACTS_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateGeneralContracts = (targetContract, generalContractName, generalContractAddress, updateType) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            generalContractName     = "${generalContractName}";
            generalContractAddress  = ("${generalContractAddress}" : address);
            updateType              = (${updateType} : updateType);
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateGeneralContracts",
            ("${targetContract}" : address)) : option(contract(updateGeneralContractsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_GENERAL_CONTRACTS_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateWhitelistTokenContracts = (targetContract, tokenContractAddress, updateType) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            tokenContractAddress  = ("${tokenContractAddress}" : address);
            updateType            = (${updateType} : updateType);
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateWhitelistTokenContracts",
            ("${targetContract}" : address)) : option(contract(updateWhitelistTokenContractsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_WHITELIST_TOKEN_CONTRACTS_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateConfig = (targetContract, targetContractType, updateConfigAction, updateConfigNewValue) => {
    // Generate the ligo return type based on the chosen contract
    var ligoReturnType;
    var ligoConfigActionType;
    switch (targetContractType) {
        case "aggregator":
            ligoReturnType = "aggregatorUpdateConfigParamsType";
            ligoConfigActionType = "aggregatorUpdateConfigActionType";
            break;
        case "aggregatorFactory":
            ligoReturnType = "aggregatorFactoryUpdateConfigParamsType";
            ligoConfigActionType = "aggregatorFactoryUpdateConfigActionType";
            break;
        case "breakGlass":
            ligoReturnType = "breakGlassUpdateConfigParamsType";
            ligoConfigActionType = "breakGlassUpdateConfigActionType";
            break;
        case "council":
            ligoReturnType = "councilUpdateConfigParamsType";
            ligoConfigActionType = "councilUpdateConfigActionType";
            break;
        case "delegation":
            ligoReturnType = "delegationUpdateConfigParamsType";
            ligoConfigActionType = "delegationUpdateConfigActionType";
            break;
        case "doorman":
            ligoReturnType = "doormanUpdateConfigParamsType";
            ligoConfigActionType = "doormanUpdateConfigActionType";
            break;
        case "emergencyGovernance":
            ligoReturnType = "emergencyUpdateConfigParamsType";
            ligoConfigActionType = "emergencyUpdateConfigActionType";
            break;
        case "farm":
            ligoReturnType = "farmUpdateConfigParamsType";
            ligoConfigActionType = "farmUpdateConfigActionType";
            break;
        case "farmFactory":
            ligoReturnType = "farmFactoryUpdateConfigParamsType";
            ligoConfigActionType = "farmFactoryUpdateConfigActionType";
            break;
        case "governance":
            ligoReturnType = "governanceUpdateConfigParamsType";
            ligoConfigActionType = "governanceUpdateConfigActionType";
            break;
        case "governanceFinancial":
            ligoReturnType = "governanceFinancialUpdateConfigParamsType";
            ligoConfigActionType = "governanceFinancialUpdateConfigActionType";
            break;
        case "governanceSatellite":
            ligoReturnType = "governanceSatelliteUpdateConfigParamsType";
            ligoConfigActionType = "governanceSatelliteUpdateConfigActionType";
            break;
        case "lendingController":
            ligoReturnType = "lendingControllerUpdateConfigParamsType";
            ligoConfigActionType = "lendingControllerUpdateConfigActionType";
            break;
        case "treasuryFactory":
            ligoReturnType = "treasuryFactoryUpdateConfigParamsType";
            ligoConfigActionType = "treasuryFactoryUpdateConfigActionType";
            break;
        case "vaultFactory":
            ligoReturnType = "vaultFactoryUpdateConfigParamsType";
            ligoConfigActionType = "vaultFactoryUpdateConfigActionType";
            break;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            updateConfigNewValue    = ${updateConfigNewValue}n; 
            updateConfigAction      = (${updateConfigAction + "(Unit)"} : ${ligoConfigActionType})
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateConfig",
            ("${targetContract}" : address)) : option(contract(${ligoReturnType}))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_CONFIG_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const pauseAll = (targetContract) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        unit,
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%pauseAll",
            ("${targetContract}" : address)) : option(contract(unit))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_PAUSE_ALL_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const unpauseAll = (targetContract) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        unit,
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%unpauseAll",
            ("${targetContract}" : address)) : option(contract(unit))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UNPAUSE_ALL_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const togglePauseEntrypoint = (targetContract, targetContractType, targetEntrypoint, pause) => {
    // Generate the ligo return type based on the chosen contract
    var ligoReturnType;
    var ligoPausableEntrypointType;
    switch (targetContractType) {
        case "aggregator":
            ligoReturnType = "aggregatorTogglePauseEntrypointType";
            ligoPausableEntrypointType = "aggregatorPausableEntrypointType";
            break;
        case "aggregatorFactory":
            ligoReturnType = "aggregatorFactoryTogglePauseEntrypointType";
            ligoPausableEntrypointType = "aggregatorFactoryPausableEntrypointType";
            break;
        case "delegation":
            ligoReturnType = "delegationTogglePauseEntrypointType";
            ligoPausableEntrypointType = "delegationPausableEntrypointType";
            break;
        case "doorman":
            ligoReturnType = "doormanTogglePauseEntrypointType";
            ligoPausableEntrypointType = "doormanPausableEntrypointType";
            break;
        case "farm":
            ligoReturnType = "farmTogglePauseEntrypointType";
            ligoPausableEntrypointType = "farmPausableEntrypointType";
            break;
        case "farmFactory":
            ligoReturnType = "farmFactoryTogglePauseEntrypointType";
            ligoPausableEntrypointType = "farmFactoryPausableEntrypointType";
            break;
        case "lendingController":
            ligoReturnType = "lendingControllerTogglePauseEntrypointType";
            ligoPausableEntrypointType = "lendingControllerPausableEntrypointType";
            break;
        case "treasury":
            ligoReturnType = "treasuryTogglePauseEntrypointType";
            ligoPausableEntrypointType = "treasuryPausableEntrypointType";
            break;
        case "treasuryFactory":
            ligoReturnType = "treasuryFactoryTogglePauseEntrypointType";
            ligoPausableEntrypointType = "treasuryFactoryPausableEntrypointType";
            break;
        case "vaultFactory":
            ligoReturnType = "vaultFactoryTogglePauseEntrypointType";
            ligoPausableEntrypointType = "vaultFactoryPausableEntrypointType";
            break;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            targetEntrypoint  = (${targetEntrypoint}(${pause ? "True" : "False"}): ${ligoPausableEntrypointType});
            empty             = Unit;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%togglePauseEntrypoint",
            ("${targetContract}" : address)) : option(contract(${ligoReturnType}))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_TOGGLE_PAUSE_ENTRYPOINT_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateWhitelistDevelopers = (targetContract, whitelistedDeveloperAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${whitelistedDeveloperAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateWhitelistDevelopers",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_WHITELIST_DEVELOPERS_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setGovernanceProxy = (targetContract, governanceProxyAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${governanceProxyAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setGovernanceProxy",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_GOVERNANCE_PROXY_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const createFarm = (targetContract, farmName, addToGeneralContracts, forceRewardFromTransfer, infinite, totalBlocks, currentRewardPerBlock, metadata, lpTokenAddress, lpTokenId, lpTokenStandard) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            name                     = "${farmName}";
            addToGeneralContracts    = ${addToGeneralContracts ? "True" : "False"};
            forceRewardFromTransfer  = ${forceRewardFromTransfer ? "True" : "False"};
            infinite                 = ${infinite ? "True" : "False"};
            plannedRewards           = record[
                totalBlocks              = (${totalBlocks}n: nat);
                currentRewardPerBlock    = ${currentRewardPerBlock}n;
            ];
            metadata                 = ("${metadata}": bytes);
            lpToken                  = record[
                tokenAddress             = ("${lpTokenAddress}" : address);
                tokenId                  = ${lpTokenId}n;
                tokenStandard            = (${lpTokenStandard == "fa12" ? "Fa12" : "Fa2"}: lpStandardType);
            ];
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%createFarm",
            ("${targetContract}" : address)) : option(contract(createFarmType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_CREATE_FARM_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const createFarmMToken = (targetContract, farmName, loanToken, addToGeneralContracts, forceRewardFromTransfer, infinite, totalBlocks, currentRewardPerBlock, metadata, lpTokenAddress, lpTokenId, lpTokenStandard) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            name                     = "${farmName}";
            loanToken                = "${loanToken}";
            addToGeneralContracts    = ${addToGeneralContracts ? "True" : "False"};
            forceRewardFromTransfer  = ${forceRewardFromTransfer ? "True" : "False"};
            infinite                 = ${infinite ? "True" : "False"};
            plannedRewards           = record[
                totalBlocks              = (${totalBlocks}n: nat);
                currentRewardPerBlock    = ${currentRewardPerBlock}n;
            ];
            metadata                 = ("${metadata}": bytes);
            lpToken                  = record[
                tokenAddress             = ("${lpTokenAddress}" : address);
                tokenId                  = ${lpTokenId}n;
                tokenStandard            = (${lpTokenStandard == "fa12" ? "Fa12" : "Fa2"}: lpStandardType);
            ];
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%createFarmMToken",
            ("${targetContract}" : address)) : option(contract(createFarmMTokenType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_CREATE_FARM_M_TOKEN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const initFarm = (targetContract, totalBlocks, currentRewardPerBlock, forceRewardFromTransfer, infinite) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            totalBlocks                 = ${totalBlocks}n;
            currentRewardPerBlock       = ${currentRewardPerBlock}n;
            forceRewardFromTransfer     = ${forceRewardFromTransfer ? "True" : "False"};
            infinite                    = ${infinite ? "True" : "False"};
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%initFarm",
            ("${targetContract}" : address)) : option(contract(initFarmParamsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_INIT_FARM_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const closeFarm = (targetContract) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        unit,
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%closeFarm",
            ("${targetContract}" : address)) : option(contract(unit))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_CLOSE_FARM_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const createTreasury = (targetContract, baker, treasuryName, addToGeneralContracts, metadata) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        (record[
            baker                   = (${baker ? "Some((\"" + baker + "\": key_hash))" : "None"} : option(key_hash));
            name                    = "${treasuryName}";
            addToGeneralContracts   = ${addToGeneralContracts ? "True" : "False"};
            metadata                = ("${metadata}": bytes);
        ] : createTreasuryType),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%createTreasury",
            ("${targetContract}" : address)) : option(contract(createTreasuryType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_CREATE_TREASURY_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const transfer = (targetContract, transfers) => {
    // Create the transfer record list
    var transfersRecord = "";
    transfers.forEach((transfer) => {
        // Create the token type
        const tokenTypeFa12 = transfer.token;
        const tokenTypeFa2 = transfer.token;
        var tokenType;
        if (transfer.token === "mav") {
            tokenType = "Mav";
        }
        else if ("fa12" in transfer.token) {
            tokenType = `Fa12(("${tokenTypeFa12.fa12}": address))`;
        }
        else if ("fa2" in transfer.token) {
            tokenType = `Fa2(record[
                    tokenContractAddress    = ("${tokenTypeFa2.fa2.tokenContractAddress}": address);
                    tokenId                 = ${tokenTypeFa2.fa2.tokenId}n;
                ])`;
        }
        // Create and append the transfer record
        transfersRecord += `
            record[
                to_       = ("${transfer.to_}" : address);
                amount    = ${transfer.amount}n;
                token     = ${tokenType};
            ];
        `;
    });
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        list[
            ${transfersRecord}
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%transfer",
            ("${targetContract}" : address)) : option(contract(transferActionType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_TRANSFER_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const mintMvnAndTransfer = (targetContract, to_, amount) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            to_ = ("${to_}" : address);
            amt = ${amount}n;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%mintMvnAndTransfer",
            ("${targetContract}" : address)) : option(contract(mintMvnAndTransferType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_MINT_MVN_AND_TRANSFER_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateTokenOperators = (targetContract, tokenContractAddress, operators) => {
    // Create the update operator list
    var operatorRecord = "";
    operators.forEach((operator) => {
        if ("addOperator" in operator) {
            const addOperatorItem = operator;
            operatorRecord += `
            Add_operator(record[
                owner    = ("${addOperatorItem.addOperator.owner}" : address);
                operator = ("${addOperatorItem.addOperator.operator}" : address);
                token_id = ${addOperatorItem.addOperator.tokenId}n;
            ]);
            `;
        }
        else {
            const removeOperatorItem = operator;
            operatorRecord += `
            Remove_operator(record[
                owner    = ("${removeOperatorItem.removeOperator.owner}" : address);
                operator = ("${removeOperatorItem.removeOperator.operator}" : address);
                token_id = ${removeOperatorItem.removeOperator.tokenId}n;
            ]);
            `;
        }
    });
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            tokenContractAddress    = ("${tokenContractAddress}" : address);
            updateOperators         = list[
            ${operatorRecord}
            ];
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateTokenOperators",
            ("${targetContract}" : address)) : option(contract(updateTokenOperatorsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_MVN_OPERATORS_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const stakeTokens = (targetContract, stakingContract, amount) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            contractAddress     = ("${stakingContract}" : address);
            amount              = ${amount}n;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%stakeTokens",
            ("${targetContract}" : address)) : option(contract(stakeTokensType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_STAKE_MVN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const unstakeTokens = (targetContract, stakingContract, amount) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record[
            contractAddress     = ("${stakingContract}" : address);
            amount              = ${amount}n;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%unstakeTokens",
            ("${targetContract}" : address)) : option(contract(unstakeTokensType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UNSTAKE_MVN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const createAggregator = (targetContract, aggregatorName, addToGeneralContracts, oraclesInformation, decimals, alphaPercentPerThousand, percentOracleThreshold, heartbeatSeconds, rewardAmountStakedMvn, rewardAmountXtz, metadata) => {
    // Create the oracle informations
    var oracleLedger = "";
    oraclesInformation.forEach((information) => {
        oracleLedger += `
            ("${information.oracleAddress}" : address) -> record [
                    oraclePublicKey = ("${information.oraclePublicKey}" : key);
                    oraclePeerId    = "${information.oraclePeerId}";
                ];`;
    });
    const lambda = `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        (record[
            name                  = "${aggregatorName}";
            addToGeneralContracts = ${addToGeneralContracts ? "True" : "False"};
            oracleLedger          = map[${oracleLedger}
            ];
            aggregatorConfig      = record [
                decimals                = ${decimals}n;
                alphaPercentPerThousand = ${alphaPercentPerThousand}n;
                percentOracleThreshold  = ${percentOracleThreshold}n;
                heartbeatSeconds        = ${heartbeatSeconds}n;
                rewardAmountStakedMvn   = ${rewardAmountStakedMvn}n;
                rewardAmountXtz         = ${rewardAmountXtz}n;
            ];
            metadata              = ("${metadata}": bytes);
        ] : createAggregatorParamsType),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%createAggregator",
            ("${targetContract}" : address)) : option(contract(createAggregatorParamsType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_CREATE_AGGREGATOR_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
    return lambda;
};
const updateInflationRate = (targetContract, inflationRate) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ${inflationRate}n,
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateInflationRate",
            ("${targetContract}" : address)) : option(contract(nat))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_INFLATION_RATE_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const triggerInflation = (targetContract) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        unit,
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%triggerInflation",
            ("${targetContract}" : address)) : option(contract(unit))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_TRIGGER_INFLATION_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const trackProductContract = (targetContract, productContractType, productContractAddress) => {
    // Choose the track entrypoint
    var trackEntrypoint;
    switch (productContractType) {
        case "aggregator":
            trackEntrypoint = "%trackAggregator";
            break;
        case "farm":
            trackEntrypoint = "%trackFarm";
            break;
        case "treasury":
            trackEntrypoint = "%trackTreasury";
            break;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${productContractAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "${trackEntrypoint}",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_TRACK_CONTRACT_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const untrackProductContract = (targetContract, productContractType, productContractAddress) => {
    // Choose the track entrypoint
    var trackEntrypoint;
    switch (productContractType) {
        case "aggregator":
            trackEntrypoint = "%untrackAggregator";
            break;
        case "farm":
            trackEntrypoint = "%untrackFarm";
            break;
        case "treasury":
            trackEntrypoint = "%untrackTreasury";
            break;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${productContractAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "${trackEntrypoint}",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UNTRACK_CONTRACT_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const addVestee = (targetContract, vesteeAddress, totalAllocatedAmount, cliffInMonths, vestingInMonths) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            vesteeAddress        = ("${vesteeAddress}" : address);
            totalAllocatedAmount = ${totalAllocatedAmount}n;
            cliffInMonths        = ${cliffInMonths}n;
            vestingInMonths      = ${vestingInMonths}n;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%addVestee",
            ("${targetContract}" : address)) : option(contract(addVesteeType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_ADD_VESTEE_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const removeVestee = (targetContract, vesteeAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${vesteeAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%removeVestee",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_REMOVE_VESTEE_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const updateVestee = (targetContract, vesteeAddress, newTotalAllocatedAmount, newCliffInMonths, newVestingInMonths) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            vesteeAddress           = ("${vesteeAddress}" : address);
            newTotalAllocatedAmount = ${newTotalAllocatedAmount}n;
            newCliffInMonths        = ${newCliffInMonths}n;
            newVestingInMonths      = ${newVestingInMonths}n;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%updateVestee",
            ("${targetContract}" : address)) : option(contract(updateVesteeType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_UPDATE_VESTEE_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const toggleVesteeLock = (targetContract, vesteeAddress) => {
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        ("${vesteeAddress}" : address),
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%toggleVesteeLock",
            ("${targetContract}" : address)) : option(contract(address))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_TOGGLE_VESTEE_LOCK_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setLoanToken = (targetContract, setLoanTokenAction) => {
    // Parse loan token action
    var loanTokenActionRecord;
    if ("createLoanToken" in setLoanTokenAction) {
        // Cast the action
        const actionCast = setLoanTokenAction;
        const createLoanTokenAction = actionCast.createLoanToken;
        // Prepare the token type
        const tokenTypeFa12 = createLoanTokenAction.tokenType;
        const tokenTypeFa2 = createLoanTokenAction.tokenType;
        var tokenType;
        if (createLoanTokenAction.tokenType === "mav") {
            tokenType = "Mav";
        }
        else if ("fa12" in createLoanTokenAction.tokenType) {
            tokenType = `Fa12(("${tokenTypeFa12.fa12}": address))`;
        }
        else if ("fa2" in createLoanTokenAction.tokenType) {
            tokenType = `Fa2(record[
                tokenContractAddress    = ("${tokenTypeFa2.fa2.tokenContractAddress}": address);
                tokenId                 = ${tokenTypeFa2.fa2.tokenId}n;
            ])`;
        }
        // Prepare the loan token record
        loanTokenActionRecord = `CreateLoanToken(record [
                tokenName                           = ("${createLoanTokenAction.tokenName}" : string);
                tokenDecimals                       = (${createLoanTokenAction.tokenDecimals}n : nat);
                oracleAddress                       = ("${createLoanTokenAction.oracleAddress}" : address);
                mTokenAddress                       = ("${createLoanTokenAction.mTokenAddress}" : address);
                reserveRatio                        = (${createLoanTokenAction.reserveRatio}n : nat);
                optimalUtilisationRate              = (${createLoanTokenAction.optimalUtilisationRate}n : nat);
                baseInterestRate                    = (${createLoanTokenAction.baseInterestRate}n : nat);
                maxInterestRate                     = (${createLoanTokenAction.maxInterestRate}n : nat);
                interestRateBelowOptimalUtilisation = (${createLoanTokenAction.interestRateBelowOptimalUtilisation}n : nat);
                interestRateAboveOptimalUtilisation = (${createLoanTokenAction.interestRateAboveOptimalUtilisation}n : nat);
                minRepaymentAmount                  = (${createLoanTokenAction.minRepaymentAmount}n : nat);
                tokenType                           = (${tokenType} : tokenType);
            ])`;
    }
    else {
        // Cast the action
        const actionCast = setLoanTokenAction;
        const updateCollateralTokenAction = actionCast.updateLoanToken;
        // Prepare the loan token record
        loanTokenActionRecord = `UpdateLoanToken(record [
                tokenName                               = ("${updateCollateralTokenAction.tokenName}" : string);
                oracleAddress                           = ("${updateCollateralTokenAction.oracleAddress}" : address);
                reserveRatio                            = (${updateCollateralTokenAction.reserveRatio}n : nat);
                optimalUtilisationRate                  = (${updateCollateralTokenAction.optimalUtilisationRate}n : nat);
                baseInterestRate                        = (${updateCollateralTokenAction.baseInterestRate}n : nat);
                maxInterestRate                         = (${updateCollateralTokenAction.maxInterestRate}n : nat);
                interestRateBelowOptimalUtilisation     = (${updateCollateralTokenAction.interestRateBelowOptimalUtilisation}n : nat);
                interestRateAboveOptimalUtilisation     = (${updateCollateralTokenAction.interestRateAboveOptimalUtilisation}n : nat);
                minRepaymentAmount                      = (${updateCollateralTokenAction.minRepaymentAmount}n : nat);
                isPaused                                = (${updateCollateralTokenAction.isPaused} : bool);
            ])`;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            action = ${loanTokenActionRecord};
            empty  = Unit;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setLoanToken",
            ("${targetContract}" : address)) : option(contract(setLoanTokenActionType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_LOAN_TOKEN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const setCollateralToken = (targetContract, setCollateralTokenAction) => {
    // Parse loan token action
    var loanTokenActionRecord;
    if ("createCollateralToken" in setCollateralTokenAction) {
        // Cast the action
        const actionCast = setCollateralTokenAction;
        const createCollateralTokenAction = actionCast.createCollateralToken;
        // Prepare the token type
        const tokenTypeFa12 = createCollateralTokenAction.tokenType;
        const tokenTypeFa2 = createCollateralTokenAction.tokenType;
        var tokenType;
        if (createCollateralTokenAction.tokenType === "mav") {
            tokenType = "Mav";
        }
        else if ("fa12" in createCollateralTokenAction.tokenType) {
            tokenType = `Fa12(("${tokenTypeFa12.fa12}": address))`;
        }
        else if ("fa2" in createCollateralTokenAction.tokenType) {
            tokenType = `Fa2(record[
                tokenContractAddress    = ("${tokenTypeFa2.fa2.tokenContractAddress}": address);
                tokenId                 = ${tokenTypeFa2.fa2.tokenId}n;
            ])`;
        }
        // Prepare the loan token record
        loanTokenActionRecord = `CreateCollateralToken(record [
                tokenName              = "${createCollateralTokenAction.tokenName}";
                tokenContractAddress   = ("${createCollateralTokenAction.tokenContractAddress}" : address);
                tokenDecimals          = ${createCollateralTokenAction.tokenDecimals}n;
                oracleAddress          = ("${createCollateralTokenAction.oracleAddress}" : address);
                protected              = ${createCollateralTokenAction.protected ? "True" : "False"};
                isScaledToken          = ${createCollateralTokenAction.isScaledToken ? "True" : "False"};
                isStakedToken          = ${createCollateralTokenAction.isStakedToken ? "True" : "False"};
                stakingContractAddress = ${createCollateralTokenAction.stakingContractAddress ? "Some((\"" + createCollateralTokenAction.stakingContractAddress + "\" : address))" : "None"};
                maxDepositAmount       = ${createCollateralTokenAction.maxDepositAmount ? "Some(" + createCollateralTokenAction.maxDepositAmount + "n)" : "None"};
                tokenType              = ${tokenType};
            ]);`;
    }
    else {
        // Cast the action
        const actionCast = setCollateralTokenAction;
        const updateLoanTokenAction = actionCast.updateCollateralToken;
        // Prepare the loan token record
        loanTokenActionRecord = `UpdateCollateralToken(record [
                tokenName              = "${updateLoanTokenAction.tokenName}";
                oracleAddress          = ("${updateLoanTokenAction.oracleAddress}" : address);
                isPaused               = ${updateLoanTokenAction.isPaused ? "True" : "False"};
                stakingContractAddress = ${updateLoanTokenAction.stakingContractAddress ? "Some((\"" + updateLoanTokenAction.stakingContractAddress + "\" : address))" : "None"};
                maxDepositAmount       = ${updateLoanTokenAction.maxDepositAmount ? "Some(" + updateLoanTokenAction.maxDepositAmount + "n)" : "None"};
            ]);`;
    }
    return `function lambdaFunction (const _ : unit) : list(operation) is
block {
    const contractOperation : operation = Mavryk.transaction(
        record [
            action = ${loanTokenActionRecord}
            empty  = Unit;
        ],
        0mav,
        case (Mavryk.get_entrypoint_opt(
            "%setCollateralToken",
            ("${targetContract}" : address)) : option(contract(setCollateralTokenActionType))) of [
                    Some(contr) -> contr
                |   None        -> (failwith("error_SET_COLLATERAL_TOKEN_THROUGH_PROXY_LAMBDA_FAIL"))
        ]
    );
} with list[contractOperation]`;
};
const getListAvailableFunctions = () => {
    console.log(setAdmin);
    console.log(setGovernance);
    console.log(setName);
    console.log(setLambda);
    console.log(setProductLambda);
    console.log(updateMetadata);
    console.log(updateWhitelistContracts);
    console.log(updateGeneralContracts);
    console.log(updateWhitelistTokenContracts);
    console.log(updateConfig);
    console.log(pauseAll);
    console.log(unpauseAll);
    console.log(togglePauseEntrypoint);
    console.log(updateWhitelistDevelopers);
    console.log(setGovernanceProxy);
    console.log(createFarm);
    console.log(createFarmMToken);
    console.log(initFarm);
    console.log(closeFarm);
    console.log(createTreasury);
    console.log(transfer);
    console.log(mintMvnAndTransfer);
    console.log(updateTokenOperators);
    console.log(stakeTokens);
    console.log(unstakeTokens);
    console.log(createAggregator);
    console.log(updateInflationRate);
    console.log(triggerInflation);
    console.log(trackProductContract);
    console.log(untrackProductContract);
    console.log(addVestee);
    console.log(removeVestee);
    console.log(updateVestee);
    console.log(toggleVesteeLock);
    console.log(setLoanToken);
    console.log(setCollateralToken);
};
const generateProxyContract = (lambdaFunctionName, lambdaFunctionParameters = []) => {
    // Create the function call
    var functionCall = lambdaFunctionName + "(";
    lambdaFunctionParameters.forEach((parameter) => {
        var formattedParameter = parameter;
        var parameterType = typeof (parameter);
        switch (parameterType) {
            case "string":
                formattedParameter = `"${formattedParameter}"`;
                break;
            case "object":
                formattedParameter = (JSON.stringify(parameter));
                break;
        }
        functionCall += formattedParameter + ",";
    });
    functionCall = functionCall.slice(0, -1) + ")";
    // Return the generated contract
    return proxyContract(eval(functionCall));
};

const packLambdaFunction = (tezos, governanceProxyContractAddress, lambdaFunction) => __awaiter(void 0, void 0, void 0, function* () {
    const governanceProxyInstance = yield tezos.contract.at(governanceProxyContractAddress);
    const param = governanceProxyInstance.methods.dataPackingHelper(lambdaFunction).toTransferParams();
    if (param.parameter) {
        const paramValue = param.parameter.value;
        const lambdaEntrypointType = yield governanceProxyInstance.entrypoints.entrypoints.dataPackingHelper;
        const packed = yield tezos.rpc.packData({
            data: paramValue,
            type: lambdaEntrypointType
        }).catch(e => console.error('error:', e));
        var packedParam;
        if (packed) {
            packedParam = packed.packed;
        }
        else {
            throw `packing failed`;
        }
    }
    return packedParam;
});
function getLigo(isDockerizedLigo, ligoVersion) {
    let path = 'ligo';
    if (isDockerizedLigo) {
        path = `docker run --platform=linux/amd64 -v $PWD:$PWD -w $PWD --rm -i mavrykdynamics/ligo:${ligoVersion}`;
        try {
            execSync(`${path}  --help`);
        }
        catch (err) {
            path = 'ligo';
            execSync(`${path}  --help`);
        }
    }
    else {
        try {
            execSync(`${path}  --help`);
        }
        catch (err) {
            path = `docker run --platform=linux/amd64 -v $PWD:$PWD -w $PWD --rm -i mavrykdynamics/ligo:${ligoVersion}`;
            execSync(`${path}  --help`);
        }
    }
    return path;
}
const compileLambdaFunctionContract = (contractPath = "", ligoVersion = "0.60.0") => __awaiter(void 0, void 0, void 0, function* () {
    const ligo = getLigo(true, ligoVersion);
    const jsonFormat = execSync(`${ligo} compile contract ${contractPath} --michelson-format json --protocol kathmandu`, {
        maxBuffer: 1024 * 1024,
        timeout: 1024 * 1024
    }).toString();
    // Get the lambda function from the compiled code
    return JSON.parse(jsonFormat)[2].args[0][1].args[0];
});
const getLambdaFunction = (rpc, governanceProxyContractAddress, lambdaFunctionName, lambdaFunctionParameters = []) => __awaiter(void 0, void 0, void 0, function* () {
    // Read the contents of the input file specified in the first command line argument
    const generatedContract = generateProxyContract(lambdaFunctionName, lambdaFunctionParameters);
    // Write the result to the output file
    const outputFile = __dirname + "/ligo/governanceProxyLambdaFunction.ligo";
    fs.writeFileSync(outputFile, generatedContract);
    // Start the compiling process
    const lambdaFunctionJson = yield compileLambdaFunctionContract(outputFile);
    const tezos = new TezosToolkit(rpc);
    const packedLambdaFunction = yield packLambdaFunction(tezos, governanceProxyContractAddress, lambdaFunctionJson);
    // Reset the file
    fs.writeFileSync(outputFile, "");
    return packedLambdaFunction;
});

(function compileLambdaFunction() {
    return __awaiter(this, void 0, void 0, function* () {
        const rpc = process.argv[2];
        const governanceProxyContractAddress = process.argv[3];
        const lambdaFunctionName = process.argv[4];
        var args = process.argv.slice(5);
        const lambdaFunctionParameters = args;
        if (!rpc || !governanceProxyContractAddress || !lambdaFunctionName) {
            getListAvailableFunctions();
            return console.info("Usage: proposal-data-maker <rpc_url> <governance_proxy_contract_address> <lambda_function_name> <parameters...>");
        }
        // Turn strings into JSON in parameters
        for (let i = 0; i < args.length; i++) {
            try {
                const fixedJSON = args[i].replace(/(['"])?([a-z0-9A-Z_]+)(['"])?:/g, '"$2": ');
                const parsedParam = JSON.parse(fixedJSON);
                args[i] = parsedParam;
            }
            catch (e) { }
        }
        // Get the lambda function
        const lambdaFunction = yield getLambdaFunction(rpc, governanceProxyContractAddress, lambdaFunctionName, lambdaFunctionParameters);
        console.log(lambdaFunction);
    });
})();
const createLambdaBytes = (rpc, governanceProxyContractAddress, lambdaFunctionName, lambdaFunctionParameters) => getLambdaFunction(rpc, governanceProxyContractAddress, lambdaFunctionName, lambdaFunctionParameters);

export { createLambdaBytes };
//# sourceMappingURL=create-lambda-bytes.es6.js.map
